package FirstBean;

import java.awt.*;
import java.beans.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Vector;

public class FirstBean extends Canvas implements Serializable,
				MouseListener, MouseMotionListener {
      private boolean down;
      private Vector pushListeners = new Vector();
	String ourString = "Hello";
	private PropertyChangeSupport changes = new PropertyChangeSupport(this);

	public FirstBean() { 
		setBackground(Color.red); setForeground(Color.blue);
		addMouseListener(this);
		addMouseMotionListener(this);
	}
	public void setString(String newString) { 
		String oldString = ourString; ourString = newString; 
		sizeToFit();
		changes.firePropertyChange("string",oldString,newString);
	}
	public String getString() { return ourString; }
	public void addPropertyChangeListener(PropertyChangeListener l)  { 
		changes.addPropertyChangeListener(l); 
	}
	public void removePropertyChangeListener( PropertyChangeListener l)  { 
		changes.removePropertyChangeListener(l); 
	}

    public synchronized void paint(Graphics g) {
	int width = getSize().width;
	int height = getSize().height;

	g.setColor(getBackground());
	g.fill3DRect(0, 0, width - 1, height - 1, !down);

	g.setColor(getForeground());
	g.setFont(getFont());

	g.drawRect(2, 2, width - 4, height - 4);

	FontMetrics fm = g.getFontMetrics();
	g.drawString(ourString, (width - fm.stringWidth(ourString)) / 2, 
		          (height + fm.getMaxAscent() - fm.getMaxDescent()) / 2);
    }
	
    public synchronized void addActionListener(ActionListener l) {
	pushListeners.addElement(l);
    }

    public synchronized void removeActionListener(ActionListener l) {
	pushListeners.removeElement(l);
    }

    public void mouseClicked(MouseEvent evt) {
    }

    public void mousePressed(MouseEvent evt) {
	if (!isEnabled()) {
	    return;
	}
	down = true;
	repaint();
    }

    public void mouseReleased(MouseEvent evt) {
	if (!isEnabled()) {
	    return;
	}
	if (down) {
	    fireAction();
	    down = false;
	    repaint();
	}
    }

    public void mouseEntered(MouseEvent evt) {
    }

    public void mouseExited(MouseEvent evt) {
    }

    public void mouseDragged(MouseEvent evt) {
	if (!isEnabled()) {
	    return;
	}
	// Has the mouse been dragged outside the button?
	int x = evt.getX();
	int y = evt.getY();
	int width = getSize().width;
	int height = getSize().height;
	if (x < 0 || x > width || y < 0 || y > height) {
	    // Yes, we should deactivate any pending click.
	    if (down) {
		down = false;
		repaint();
	    }
	} else if (!down) {
	    down = true;
	    repaint();
	}
    }

    public void mouseMoved(MouseEvent evt) {
    }

    public void fireAction() {
/*	if (debug) {
	    System.err.println("Button " + getLabel() + " pressed.");
	}*/
	Vector targets;
	synchronized (this) {
	    targets = (Vector) pushListeners.clone();
	}
	ActionEvent actionEvt = new ActionEvent(this, 0, null);
	for (int i = 0; i < targets.size(); i++) {
	    ActionListener target = (ActionListener)targets.elementAt(i);
	    target.actionPerformed(actionEvt);
	}

    }

    public Dimension getPreferredSize() {
	FontMetrics fm = getFontMetrics(getFont());
	return new Dimension(fm.stringWidth(ourString) + TEXT_XPAD, 
			     fm.getMaxAscent() + fm.getMaxDescent() + TEXT_YPAD);
    }

    public Dimension preferredSize() {
	return getPreferredSize();
    }

    public Dimension getMinimumSize() {
	return getPreferredSize();
    }

    private void sizeToFit() {
	Dimension d = getPreferredSize();
	setSize(d.width, d.height);
	Component p = getParent();
	if (p != null) {
	    p.invalidate();
	    p.doLayout();
	}
    }

    static final int TEXT_XPAD = 12;
    static final int TEXT_YPAD = 8;
}
