package sunw.demo.listview;

import java.util.Vector;
import java.awt.*;
import java.awt.event.*;

public class ListItem extends Object
{
  Image itemIcon;
  int itemIndex;
  String itemTitle;
  boolean haveDetail;
  Vector itemDetails;
  int detailCount;

  private Component Parent;
  private Dimension size;
  private boolean pressed;
  private boolean mouseIn;
  private int x, y;

  // Class initialization code
  ListItem(Component parent, Image icon, int index, String title) {
    this(icon, index, title, true);
  }

  ListItem(Component parent, Image icon, int index, String title, boolean detail) {
    Parent = parent;
    FontMetrics fm = Parent.getFontMetrics(Parent.getFont());

    int height  = (itemIcon.getHeight(Parent) > fm.getHeight()) ?
                       itemIcon.getHeight(Parent) : fm.getHeight();
    height += 2;
    int width = 4 + itemIcon.getWidth(Parent) + fm.stringWidth(itemTitle);
    itemIcon = icon;
    itemIndex = index;
    itemTitle = title;
    haveDetail = detail;
    itemDetails = new Vector();
    detailCount = 0;
    size = new Dimension(height, width);
  }

  public int getItemIndex() { return itemIndex; };

  public void setItemIndex(int index) { itemIndex = index; };

  public Image getItemIcon() { return itemIcon; };

  public void setItemIcon(Image icon) { itemIcon = icon; };

  public String getItemTitle() { return itemTitle; };

  public void setItemTitle(String title) { itemTitle = title; };

  public int getDetailCount() { return detailCount; };

  public void setDetailCount(int count) { detailCount = count; };

  public Dimension getSize() { return size; };

  public void setSize(Dimension sz) { size = sz; };

  public void addItemDetail(ListItemDetail detail) {
    itemDetails.addElement(detail);
    detailCount = itemDetails.size();
  }

  public boolean removeItemDetail(ListItemDetail detail) {
    itemDetails.removeElement(detail);
    detailCount = itemDetails.size();
  }

  public void setOrigin(Point origin)
  {
    x = origin.x;
    y = origin.y;
  }

  public boolean mouseEnter(MouseEvent event)
  {
    mouseIn = true;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseExit(MouseEvent event)
  {
    mouseIn = false;
//    pressed = false;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseDown(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (mouseIn)
        pressed = true;
    repaint();
    return true;
  }

  // Mouse button released handling code
  public boolean mouseUp(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (pressed && mouseIn) {
         Vector targets;
         synchronized (this) {
            targets = (Vector) ((ListView)Parent).pushListeners.clone();
         }
	   ActionEvent actionEvt = new ActionEvent(menuEq,
                         ActionEvent.ACTION_PERFORMED, itemTitle);
         for (int i = 0; i < targets.size(); i++) {
            ActionListener target = (ActionListener)targets.elementAt(i);
            target.actionPerformed(actionEvt);
         }
         pressed = false;
         repaint();
      }
    return true;
  }

  public boolean hasPoint(int x, int y)
  {
    if (x > this.x && x < this.x + size.width)
      if (y > this.y && y < this.y + size.height)
        return true;
    return false;
  }

  public void repaint()
  {
    paint(Parent.getGraphics());
  }

  // This routine handles the painting
  public void paint(Graphics graphic)
  {
    if (itemIcon != null) 
    {
      if (mouseIn)
      {
        graphic.setColor(Color.black);
        graphic.drawRect(x + 1, y + 1, size.width - 1, size.height - 1);
        graphic.drawImage(bitmap, x + 1, y + 1, itemIcon.getWidth(Parent),
                          itemIcon.getHeight(Parent), Parent);
        graphic.drawString(itemTitle, x + itemIcon.getWidth(Parent) + 2,
                           y + 2);
      }
      else
      {
        graphic.setColor(Color.black);
        graphic.drawImage(bitmap, x + 1, y + 1, itemIcon.getWidth(Parent),
                          itemIcon.getHeight(Parent), Parent);
        graphic.drawString(itemTitle, x + itemIcon.getWidth(Parent) + 2,
                           y + 2);
      } 
   }
  }
} // end class ListItem
